#include "common.h"

int debug = 0;

static void
init_daemon()
{
        int ret;

        openlog("smsotpd", LOG_PID, LOG_DAEMON);

        ret = chdir("/");
        if (ret < 0) {
                die("%s", "chdir");
        }

        ret = fork();
        if (ret < 0) {
                die("%s", "fork");

        } else if (ret) {
                exit(0);
        }

        ret = setsid();
        if (ret < 0) {
                die("%s", "setsid");
        }
        ret = close(0);
        if (ret < 0) {
                die("%s", "close");
        }
        ret = close(1);
        if (ret < 0) {
                die("%s", "close");
        }
        ret = close(2);
        if (ret < 0) {
                die("%s", "close");
        }
}

static void
check_for_existing_instance()
{
        int fd, ret, pid;
        char buf[BUFSIZ];

        fd = open(PIDFILE, O_RDONLY);
        if (fd < 0) {
                return;
        }

        ret = lseek(fd, 0, SEEK_SET);
        if (ret < 0) {
                die("%s", "lseek");
        }

        ret = read(fd, buf, sizeof(buf));
        if (ret < 0) {
                die("%s", "read");
        }

        pid = atoi(buf);

        ret = close(fd);
        if (ret < 0) {
                die("%s", "close");
        }

        ret = kill(pid, 0);
        if (ret == 0) {
                die("smsotpd is already running with pid %d. Exiting.", pid);
        }

        ret = unlink(PIDFILE);
        if (ret < 0) {
                die("unable to remove pidfile '%s'", PIDFILE);
        }
}

static void
create_pid_file()
{
        int fd, ret, len;
        char buf[BUFSIZ];

        fd = open(PIDFILE, O_EXCL | O_CREAT | O_WRONLY, 0600);
        if (fd < 0) {
                die("cannot create pid file '%s'", PIDFILE);
        }

        len = snprintf(buf, BUFSIZ, "%d\n", getpid());

        ret = write(fd, buf, len);
        if (ret < 0) {
                die("Can't write pid to %s", PIDFILE);
        }

        ret = close(fd);
        if (ret < 0) {
                die("%s", "close");
        }
}

int
main(int argc, char **argv)
{
        int listen_socket, client_socket, client_len;
        struct sockaddr_un client_addr;

        char *input_file = NULL;
        int c;

        opterr = 0;

        while ((c = getopt(argc, argv, "df:")) != -1) {
                switch (c) {
                case 'd':
                        debug = 1;
                        break;
                case 'f':
                        input_file = optarg;
                        break;
                case '?':
                        if (optopt == 'f') {
                                fprintf (stderr, "Option -%c requires an argument. Aborting\n", optopt);
                                exit(-1);
                        }
                        fprintf(stderr, "Syntax: %s [-d] [-f <file>]\n", argv[0]);
                        fprintf(stderr, "\t-d        - Debug\n");
                        fprintf(stderr, "\t-f <file> - Generate Berkeley database (username<tab>mobile)\n");
                        return 1;
                default:
                        abort();
                }
        }

        if (input_file) {
                create_berkley_db(input_file);
                return 0;
        }

        if (! debug) {
                init_daemon();
        }

        check_for_existing_instance();
        create_pid_file();

        db_open(DATABASE, DB_RDONLY);

        signal(SIGPIPE, SIG_IGN);
        signal(SIGCHLD, SIG_IGN);

        listen_socket = server_listen_unix_socket(SOCKET);

        srand(time(NULL));

        client_len = sizeof(client_addr);

        while (1) {
                memset(&client_addr, 0, sizeof(struct sockaddr_un));
                client_socket = accept(listen_socket,
                                (struct sockaddr *) &client_addr, (socklen_t *) &client_len);
                if (client_socket < 0) {
                        die("%s", "accept failed");
                }

                dprintf("Client connected\n");

                handle_client(client_socket);

                dprintf("Client disconnected\n");
        }

        close(listen_socket);
        db_close();

        return 0;
}
